/*
 * CMUPA.h
 * Author: Yun-Ching Lee (ycl+@cmu.edu)
 * Copyright 1998 by Carnegie Mellon University
 * All rights reserved
 */

// CMUPA.h: interface for the CCMUPA class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_CMUPA_H__7D03CCB1_1500_11D2_8AA9_006008A82114__INCLUDED_)
#define AFX_CMUPA_H__7D03CCB1_1500_11D2_8AA9_006008A82114__INCLUDED_

#if _MSC_VER >= 1000
#pragma once
#endif // _MSC_VER >= 1000

#include <sys/types.h>
#include "enums.h"

/*
 * CCMUPA class interfaces:
 * int pack_cmupa(char *):	packs the internal structure to a character stream
 *							returns the length of character stream or ERROR (-1)
 * int unpack_cmupa(char *):unpacks the character stream to in the internal structure
 *							returns the length of character stream or ERROR (-1)
 * CMUPA data:				The data structure
 */

/*
 * this header file defines a huge typed CMUPA-like structure
 * 6/6/98
 */

typedef unsigned long u_long;
typedef unsigned short u_short;

#define	MAX_BUFFER_LENGTH	262144
#define	TRAJECTORY_LENGTH	15
#define	SKELETON_SIZE	20
#define	BOUNDARY_SIZE	20
#define	ROI_SIZE		16
#define	HISTOGRAM_SIZE	64
#define	TARGET_MAX		16
#define	EVENT_MAX		16
#define	LABEL_MAX		32
#define	DISPAR_MAX		32
#define	BLOCK_MAX		10

#ifndef ERROR
#define	ERROR			-1
#endif

#define	TESTBIT(x,y)		((x) & (0x1 << y))
#define	SETBIT(x,y)		(x = (x | (0x1 << y)))
#define	CLEARBIT(x,y)		(x = ~(~x | (0x1 << y)))

typedef struct _bitmasks {
	u_long          c_bitmask;
	u_long          s_bitmask;
	u_long          i_bitmask;
	u_long          r_bitmask;
	u_long          e_bitmask;
}               BITMASKS;

typedef struct _image_point {
	__int16         u;
	__int16         v;
}               IMAGE_POINT;

typedef struct _image_velocity {
	double          u;
	double          v;
}               IMAGE_VELOCITY;

typedef struct _bounding_box {
	IMAGE_POINT     top_left;
	IMAGE_POINT     bottom_right;
}               BOUNDING_BOX;

typedef struct _boundary {
	__int16         length;
	IMAGE_POINT		value[BOUNDARY_SIZE];
}               BOUNDARY;

typedef struct _skeleton {
	IMAGE_POINT     centroid;
	__int16         length;
	IMAGE_POINT     extremal[SKELETON_SIZE];
}               SKELETON;

typedef struct _lvcs_trip {
	float           x;
	float           y;
	float           z;
}               LVCS_TRIP;

typedef struct _gps_trip {
	long            lat;
	long            lon;
	float           alt;
}               GPS_TRIP;

typedef struct _gps_vel {
	float           lat;
	float           lon;
	float           alt;
}               GPS_VEL;

typedef struct _mount_trip {
	long            theta;
	long            phi;
	long            rho;
}               MOUNT_TRIP;

typedef struct _mount_vel {
	float           theta;
	float           phi;
	float           rho;
}               MOUNT_VEL;

typedef struct _fov_pair {
	long            h_angle;
	long            v_angle;
}               FOV_PAIR;

typedef struct _sensor_pars {
	short           iris;
	short           focus;
	short           u_centre;
	short           v_centre;
	float           distortion;
}               SENSOR_PARS;

typedef struct _ptz_quad {
	long            pan;
	long            tilt;
	long            twist;
	float           zoom;
}               PTZ_QUAD;

typedef struct _ptz_vel {
	float           pan;
	float           tilt;
	float           twist;
	float           zoom;
}               PTZ_VEL;

typedef struct _ptz_roi {
	short           num_pos;
	PTZ_QUAD        ptz_coord[ROI_SIZE];
}               PTZ_ROI;

typedef struct _lvcs_region {
	short           num_pos;
	LVCS_TRIP       lvcs_coord[ROI_SIZE];
}               LVCS_REGION;

typedef struct _orientation {
	long            theta;
	long            phi;
}               ORIENTATION;

typedef struct _ptz_pair {
	ORIENTATION     top_left;
	ORIENTATION     bottom_right;
}               PTZ_PAIR;

typedef struct _image_block {
	u_long          time;
	u_short         width;
	u_short         height;
	u_short         format;
	u_long          size;
	unsigned char  *data;
}               IMAGE_BLOCK;

typedef struct _video_block {
	u_long          start_time;
	u_long          end_time;
	u_short         width;
	u_short         height;
	u_short         format;
	u_long          size;
	unsigned char  *data;
}               VIDEO_BLOCK;

typedef struct _covariance_mat {
	float           C[3][3];
}               COVARIANCE_MAT;

typedef struct _temp {
        IMAGE_POINT     size;
        IMAGE_POINT     offset;
        __int16         packing;
        unsigned char   * data;
}               TEMP;

typedef struct _c_histogram {
	u_short         colour_space;
	u_short         channel_1_bins;
	u_short         channel_2_bins;
	u_short         channel_3_bins;
	u_short         channel_1[HISTOGRAM_SIZE];
	u_short         channel_2[HISTOGRAM_SIZE];
	u_short         channel_3[HISTOGRAM_SIZE];
}               C_HISTOGRAM;

typedef struct _depth_res {
	u_short         disparities;
	float           res_table[DISPAR_MAX];
}               DEPTH_RES;

typedef struct _on_off_fields {
	u_short         trajectory_len;
	u_long          time[TRAJECTORY_LENGTH];
}               ON_OFF_FIELDS;





typedef struct _command_block {
	u_short         block_size;
	u_short         command_id;
	u_long          command_type;
	u_long          c_bitmask;
	u_short         num_targets;
	u_short         target_id[TARGET_MAX];
	u_short         num_events;
	u_short         event_id[EVENT_MAX];
	BITMASKS        packet_bitmask;
	u_long          bandwidth;
}               COMMAND_BLOCK;

typedef struct _sensor_block {
	u_short         block_size;
	u_short         sensor_id;
	u_short         sensor_type;
	u_short         sensor_status;
	u_short         label_len;
	char            label[LABEL_MAX];
	u_long          s_bitmask;
	LVCS_TRIP       lvcs_pos;
	LVCS_TRIP       lvcs_err;
	LVCS_TRIP       lvcs_vel;
	GPS_TRIP        gps_pos;
	GPS_TRIP        gps_err;
	GPS_VEL         gps_vel;
	MOUNT_TRIP      mount_pos;
	MOUNT_TRIP      mount_err;
	MOUNT_VEL       mount_vel;
	IMAGE_POINT     image_size;
	FOV_PAIR        fov;
	SENSOR_PARS     sen_pars;
	PTZ_QUAD        ptz_pos;
	PTZ_VEL         ptz_vel;
	PTZ_ROI         ptz_roi;
	LVCS_REGION     lvcs_roi;
	PTZ_PAIR        ptz_fp;
	LVCS_REGION     lvcs_fp;
	PTZ_PAIR        ptz_lim;
	LVCS_REGION     lvcs_lim;
}               SENSOR_BLOCK;

typedef struct _imagery_block {
	u_long          block_size;
	u_short         imagery_id;
	u_short         imagery_type;
	u_short         label_len;
	char            label[LABEL_MAX];
	u_long          i_bitmask;
	IMAGE_BLOCK     imagery;
	VIDEO_BLOCK     video;
}               IMAGERY_BLOCK;

typedef struct _target_block {
	u_short         block_size;
	u_short         target_id;
	u_short         target_type;
	u_short         label_len;
	char            label[LABEL_MAX];
	u_long          r_bitmask;
	u_long          t_bitmask;
	IMAGE_POINT     img_pos;
	LVCS_TRIP       lvcs_pos;
	COVARIANCE_MAT  lvcs_err;
	IMAGE_VELOCITY  img_vel;
	LVCS_TRIP       lvcs_vel;
	BOUNDING_BOX    bounding_box;
	SKELETON        skeleton;
	BOUNDARY        boundary;
	u_short         target_size;
	C_HISTOGRAM     histogram;
	TEMP            temp;
	TEMP            depth_map;
	DEPTH_RES       depth_res;
	u_short         behav;
	ON_OFF_FIELDS   on_off;
	IMAGE_POINT     traj_img_pos[TRAJECTORY_LENGTH];
	LVCS_TRIP       traj_lvcs_pos[TRAJECTORY_LENGTH];
	COVARIANCE_MAT  traj_lvcs_err[TRAJECTORY_LENGTH];
	BOUNDING_BOX    traj_bound_box[TRAJECTORY_LENGTH];
	SKELETON        traj_skeleton[TRAJECTORY_LENGTH];
	BOUNDARY        traj_boundary[TRAJECTORY_LENGTH];
	u_short         traj_size[TRAJECTORY_LENGTH];
	C_HISTOGRAM     traj_histogram[TRAJECTORY_LENGTH];
	TEMP            traj_template[TRAJECTORY_LENGTH];
	TEMP            traj_depth_map[TRAJECTORY_LENGTH];
	u_short         traj_behav[TRAJECTORY_LENGTH];
}               TARGET_BLOCK;

typedef struct _event_block {
	u_short         block_size;
	u_short         event_id;
	u_short         event_type;
	u_short         label_len;
	char            label[LABEL_MAX];
	u_long          e_bitmask;
	IMAGE_POINT     event_img_pos;
	float           event_img_range;
	u_long          event_img_time;
	LVCS_TRIP       event_lvcs_pos;
	u_long          event_lvcs_time;
	u_short         num_targets;
	u_short         target_id[TARGET_MAX];
}               EVENT_BLOCK;

typedef struct _cmupa {
	u_long          packet_size;
	u_long          packet_id;
	u_short         scenario_id;
	u_short         source_id;
	u_short         dest_id;
	u_long          time;
	u_short         h_bitmask;
	u_short         num_commands;
	COMMAND_BLOCK   command_block[BLOCK_MAX];
	u_short         num_sensors;
	SENSOR_BLOCK    sensor_block[BLOCK_MAX];
	u_short         num_imageries;
	IMAGERY_BLOCK   imagery_block[BLOCK_MAX];
	u_short         num_targets;
	TARGET_BLOCK    target_block[BLOCK_MAX];
	u_short         num_events;
	EVENT_BLOCK     event_block[BLOCK_MAX];
}               CMUPA;

typedef union _float_bitmask {
	float fpnum;
	u_long intnum;
} FLOAT_BITMASK;


class CCMUPA  
{
public:
	CMUPA data;
	long pack_cmupa(char *);
	long unpack_cmupa(char *);
	CCMUPA();
	virtual ~CCMUPA();

private:
	long buftofloat(char *, float *);
	long buftoint32(char *, u_long *);
	long buftoint32(char * buffer, long * integer ) { return buftoint32(buffer, (u_long *) integer); };
	long buftoint16(char *, u_short *);
	long buftoint16(char * buffer, short * integer ) { return buftoint16(buffer, (u_short *) integer); };
	long floattobuf(float, char *);
	long int32tobuf(u_long, char *);
	long int32tobuf(long integer, char * buffer) { return int32tobuf((u_long) integer, buffer); };
	long int16tobuf(u_short, char *);
	long int16tobuf(short integer, char * buffer) { return int16tobuf((u_short) integer, buffer); };
	long pack_event(EVENT_BLOCK *, char *);
	long pack_target(TARGET_BLOCK *, char *);
	long pack_imagery(IMAGERY_BLOCK *, char *);
	long pack_sensor(SENSOR_BLOCK *, char *);
	long pack_command(COMMAND_BLOCK *, char *);
	long unpack_event(char *, EVENT_BLOCK *);
	long unpack_target(char *, TARGET_BLOCK *);
	long unpack_imagery(char *, IMAGERY_BLOCK *);
	long unpack_sensor(char *, SENSOR_BLOCK *);
	long unpack_command(char *, COMMAND_BLOCK *);
};

#endif // !defined(AFX_CMUPA_H__7D03CCB1_1500_11D2_8AA9_006008A82114__INCLUDED_)
