/*
 * Comms.cpp
 * Author: Yun-Ching Lee (ycl+@cmu.edu)
 * Copyright 1998 by Carnegie Mellon University
 * All rights reserved
 */

// Comms.cpp: implementation of the CComms class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
//#include "classcmupa.h"
#include "Comms.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CComms::CComms()
{
	m_initialized = FALSE;
	m_bandwidth =  4294967295;
	m_lasttick = GetTickCount();
	SPUID = -1;
}

CComms::~CComms()
{

}

BOOL CComms::Connect(char * hostaddr, int hostport)
{
	WSADATA WSAData;

	if(SPUID < 0)
		return FALSE;

	m_initialized = FALSE;
	m_bandwidth =  4294967295;
	m_lasttick = GetTickCount();
	m_shutup = FALSE;

	WSAStartup(0x0200, &WSAData);

	m_sai.sin_family = AF_INET;
	m_sai.sin_port = htons(hostport);
	m_sai.sin_addr.s_addr = inet_addr(hostaddr);

	m_sock = socket(AF_INET, SOCK_STREAM, 0);
	if(m_sock == INVALID_SOCKET)
		return FALSE;

	if(connect(m_sock, (struct sockaddr *)&m_sai, sizeof(m_sai)) == SOCKET_ERROR)
		return FALSE;

	u_long yes = 1;
	ioctlsocket(m_sock, FIONBIO, &yes);
	m_initialized = TRUE;


	/* put handshaking stuff here */
	/* send I AM ALIVE */
	OutPacket.data.packet_size = 20 + 14;
	OutPacket.data.packet_id = 0;
	OutPacket.data.scenario_id = 0;
	OutPacket.data.source_id = SPUID;
	OutPacket.data.dest_id = ID_OCU_1;
	OutPacket.data.time = 0;
	OutPacket.data.h_bitmask = 0x1;
	OutPacket.data.num_commands = 1;
	OutPacket.data.command_block[0].block_size = 12;
	OutPacket.data.command_block[0].command_id = 0;
	OutPacket.data.command_block[0].command_type = CM_I_AM_ALIVE;
	OutPacket.data.command_block[0].c_bitmask = 0;
	SendNow();

	/* send bitmask of supported */
	/* ideally, we want the SPU to set bitmasks and sensor status before calling Connect() */
	/* and we only need to set the header bitmask before sending */
	OutPacket.data.packet_size = 20 + 38;
	OutPacket.data.packet_id = 0;
	OutPacket.data.scenario_id = 0;
	OutPacket.data.source_id = SPUID;
	OutPacket.data.dest_id = ID_OCU_1;
	OutPacket.data.time = 0;
	OutPacket.data.h_bitmask = 0x3;	/* command and sensor block */
	OutPacket.data.num_commands = 1;
	OutPacket.data.command_block[0].block_size = 36;
	OutPacket.data.command_block[0].command_id = 0;
	OutPacket.data.command_block[0].command_type = CM_SET_PACKET_DESCRIPTION;
	OutPacket.data.command_block[0].c_bitmask = 0x4;
	OutPacket.data.num_sensors = 1;
	SendNow();

	m_lasttick = GetTickCount();

	return TRUE;
}

BOOL CComms::Disconnect()
{
	if(!m_initialized)
		return TRUE;

/*
	// compiler does not recognize SD_BOTH, which is in WinSock2.h...  argh!
	if(shutdown(m_sock, SD_BOTH) == SOCKET_ERROR)
		return FALSE;

	while(recv(m_sock, NULL, 4096, 0));
*/
	if(closesocket(m_sock) == SOCKET_ERROR);
		return FALSE;

	m_initialized = FALSE;
	return TRUE;
}

BOOL CComms::Send()
{
	char outbuffer[MAX_BUFFER_LENGTH];
	long length;
	long pos = 0;
	long remain;
	long sent;

	if(!m_initialized)
		return FALSE;

	if(m_shutup)
		return FALSE;

	length = OutPacket.pack_cmupa(outbuffer);
	if(length == ERROR)
		return FALSE;

	u_long tickstowait = max(250, (length * 1000) / m_bandwidth);
	u_long ticksince = GetTickCount() - m_lasttick;
	if(ticksince < tickstowait)
		Sleep(tickstowait - ticksince);

	remain = length;
	while(remain > 0) {
		sent = send(m_sock, outbuffer+pos, remain, 0);
		if(sent > 0) {
			pos += sent;
			remain -= sent;
		}
	}

	m_lasttick = GetTickCount();

	return TRUE;
}

BOOL CComms::SendNow()
{
	char outbuffer[MAX_BUFFER_LENGTH];
	long length;
	long pos = 0;
	long remain;
	long sent;

	if(!m_initialized)
		return FALSE;

	if(m_shutup)
		return FALSE;

	length = OutPacket.pack_cmupa(outbuffer);
	if(length == ERROR)
		return FALSE;

	remain = length;
	while(remain > 0) {
		sent = send(m_sock, outbuffer+pos, remain, 0);
		if(sent > 0) {
			pos += sent;
			remain -= sent;
		}
	}

	m_lasttick = GetTickCount();

	return TRUE;
}

BOOL CComms::Receive()
{
	char inbuffer[MAX_BUFFER_LENGTH];
	long length;
	long pos = 0;
	long remain;
	long received;
	int retval;

	if(!m_initialized)
		return FALSE;

	received = recv(m_sock, inbuffer, 4, 0);
	if(received != 4)
		return FALSE;
	length = ntohl(*(u_long *)&inbuffer);
	remain = length - 4;

	while(remain > 0) {
		received = recv(m_sock, inbuffer+pos, remain, 0);
		if(received > 0) {
			pos += received;
			remain -= received;
		}
	}

	retval = InPacket.unpack_cmupa(inbuffer);

	if(retval == ERROR)
		return FALSE;
	return TRUE;
}
