/*
 * Comms.h
 * Author: Yun-Ching Lee (ycl+@cmu.edu)
 * Copyright 1998 by Carnegie Mellon University
 * All rights reserved
 */

// Comms.h: interface for the CComms class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_COMMS_H__F6684674_1502_11D2_8AA9_006008A82114__INCLUDED_)
#define AFX_COMMS_H__F6684674_1502_11D2_8AA9_006008A82114__INCLUDED_

#include "cmupa.h"	// Added by ClassView
#if _MSC_VER >= 1000
#pragma once
#endif // _MSC_VER >= 1000

#include "enums.h"

/*
 * CComms class interfaces:
 * all functions return TRUE if successful or FALSE if unsuccessful
 * BOOL Connect(char *, int):	initialize and connect to the OCU and perform handshaking
 * BOOL Send(void):		send a packet to OCU, following the bandwidth guideline
 * BOOL SendNow(void):		send a packet to OCU immediately
 * BOOL Receive(void):		receive a packet from OCU
 * BOOL Disconnect(void):	disconnect from OCU and reinitialize all variables
 * CCMUPA InPacket:		data structures for sending and receiving packets to
 * CCMUPA OutPacket:		and from OCU
 * int SPUID:			Enumerated SPU identifier (appendix F)
 *				must be supplied prior to Connect
 */

class CComms  
{
public:
	int SPUID;
	CCMUPA OutPacket;
	CCMUPA InPacket;
	CComms();
	virtual ~CComms();
	BOOL Receive(void);
	BOOL SendNow(void);
	BOOL Send(void);
	BOOL Disconnect(void);
	BOOL Connect(char *, int);
	BOOL m_initialized;
private:
	BOOL m_shutup;
	struct sockaddr_in m_sai;
	SOCKET m_sock;
	u_long m_bandwidth;
	u_long m_lasttick;
};

#endif // !defined(AFX_COMMS_H__F6684674_1502_11D2_8AA9_006008A82114__INCLUDED_)
