/*
 * enums.h - enumerations for CMUPA
 * Author: Yun-Ching Lee (ycl+@cmu.edu)
 * Copyright 1998 by Carnegie Mellon University
 * All rights reserved
 */


#ifndef __ENUMS_H__
#define __ENUMS_H__

/* bitmask enumerations */
#define	H_MASK_COMMAND			0x0001
#define	H_MASK_SENSOR			0x0002
#define	H_MASK_IMAGERY			0x0004
#define	H_MASK_TARGET			0x0008
#define H_MASK_EVENT			0x0010

#define	C_MASK_TARGET_FIELD		0x00000001
#define	C_MASK_EVENT_FIELD		0x00000002
#define	C_MASK_BITMASK			0x00000004
#define	C_MASK_BANDWIDTH		0x00000008

#define	S_MASK_LVCS_POS			0x00000001
#define	S_MASK_LVCS_ERR			0x00000002
#define	S_MASK_LVCS_VEL			0x00000004
#define	S_MASK_GPS_POS			0x00000008
#define	S_MASK_GPS_ERR			0x00000010
#define	S_MASK_GPS_VEL			0x00000020
#define	S_MASK_MOUNT_POS		0x00000040
#define	S_MASK_MOUNT_ERR		0x00000080
#define	S_MASK_MOUNT_VEL		0x00000100
#define	S_MASK_IMAGE_SIZE		0x00000200
#define	S_MASK_FOV			0x00000400
#define	S_MASK_INT_PARS			0x00000800
#define	S_MASK_PTZ_POS			0x00001000
#define	S_MASK_PTZ_VEL			0x00002000
#define	S_MASK_PTZ_ROI			0x00004000
#define	S_MASK_LVCS_ROI			0x00008000
#define	S_MASK_PTZ_FP			0x00010000
#define	S_MASK_LVCS_FP			0x00020000
#define	S_MASK_PTZ_LIM			0x00040000
#define	S_MASK_LVCS_LIM			0x00080000

#define	I_MASK_IMAGE			0x00000001
#define	I_MASK_VIDEO			0x00000002

#define	R_MASK_IMG_POS			0x00000001
#define	R_MASK_LVCS_POS			0x00000002
#define	R_MASK_IMG_ERR			0x00000004
#define	R_MASK_IMG_VEL			0x00000008
#define	R_MASK_LVCS_VEL			0x00000010
#define	R_MASK_BOUND_BOX		0x00000020
#define	R_MASK_SKELETON			0x00000040
#define	R_MASK_BOUNDARY			0x00000080
#define	R_MASK_SIZE			0x00000100
#define	R_MASK_HISTOGRAM		0x00000200
#define	R_MASK_TEMPLATE			0x00000400
#define	R_MASK_DEPTH_MAP		0x00000800
#define	R_MASK_DEPTH_RES		0x00001000
#define	R_MASK_BEHAV			0x00002000

#define	T_MASK_ON_OFF			0x00000001
#define	T_MASK_IMG_POS			0x00000002
#define	T_MASK_LVCS_POS			0x00000004
#define	T_MASK_LVCS_ERR			0x00000008
#define	T_MASK_BOUND_BOX		0x00000010
#define	T_MASK_SKELETON			0x00000020
#define	T_MASK_BOUNDARY			0x00000040
#define	T_MASK_SIZE			0x00000080
#define	T_MASK_HISTOGRAM		0x00000100
#define	T_MASK_TEMPLATE			0x00000200
#define	T_MASK_DEPTH_MAP		0x00000400
#define	T_MASK_BEHAV			0x00000800

#define	E_MASK_IMG_POS			0x00000001
#define	E_MASK_LVCS_POS			0x00000002
#define	E_MASK_TARGETS			0x00000004

/* command enumerations */
#define	CM_I_AM_ALIVE			0x00000001
#define	CM_SET_BANDWIDTH		0x00000002
#define CM_SHUT_UP			0x00000004
#define CM_GENERAL_ACK			0x00000008
#define CM_NET_ERROR			0x00000010
#define CM_REQUEST_DATA			0x00000020
#define CM_REQUEST_DATA_PRIORITY	0x00000040
#define CM_SET_DATA			0x00000080
#define CM_SET_PACKET_DESCRIPTION	0x00000100
#define CM_NO_OPERATION			0x00000200
#define CM_NO_RESPONSE			0x00000400
#define CM_PARTIAL_RESPONSE		0x00000800
#define CM_FULL_RESPONSE		0x00001000
#define CM_SET_IDLE_POS			0x00002000
#define CM_RESET_IDLE_POS		0x00004000
#define CM_SET_IDLE_BEHAV		0x00008000
#define CM_RESET_IDLE_BEHAV		0x00010000
#define CM_SET_HANDOFF			0x00020000
#define CM_RESET_HANDOFF		0x00040000
#define CM_NEW_TARGET			0x00080000
#define CM_TRACK_TARGET			0x00100000
#define CM_STOP_TRACK_TARGET		0x00200000
#define CM_SEARCH_TARGET		0x00400000
#define CM_STOP_SEARCH_TARGET		0x00800000
#define CM_SEARCH_EVENT			0x01000000
#define CM_STOP_SEARCH_EVENT		0x02000000

/* sensor enumerations */
#define TY_UNCLASSIFIED			0
#define TY_RGB_VIDEO			1
#define TY_MONO_VIDEO			2
#define TY_IR_VIDEO			3
#define TY_THERMAL_VIDEO		4
#define TY_IFD_SPU_TYPE_I		5
#define TY_IFD_SPU_TYPE_II		6
#define TY_IFD_SPU_TYPE_III		7
#define TY_IFD_SPU_AIRBORNE		8
#define TY_TI_SPU			9
#define TY_OMNI_SPU			10
#define TY_SVM_SPU			11

/*
 * redundant definitions use entity ids instead #define ID_SPU_1
 * #define ID_SPU_2			1 #define ID_SPU_3
 * #define ID_SPU_4			3 #define ID_SPU_5
 * #define ID_SPU_6			5 #define ID_SPU_7
 * #define ID_SPU_8			7 #define ID_SPU_9
 * #define ID_SPU_10			9
 */

#define STAT_STANDBY			0
#define STAT_IDLE			1
#define STAT_TRACKING			2
#define STAT_MULTI_TRACKING		3
#define STAT_SEARCHING			4

/* imagery enumerations */
#define IMG_RAW_MONO			0
#define IMG_RAW_RGB			1
#define IMG_RAW_RGBA			2
#define IMG_RAW_ABGR			3
#define IMG_GIF				4
#define IMG_PGM				5
#define IMG_JPEG			6

#define MV_AVI				0
#define MV_QT				1
#define MV_MJPEG			2
#define MV_MPEG				3
#define MV_SGI				4

/* target enumerations */
#define TG_UNCLASSIFIED			0
#define TG_HUMAN			1
#define TG_HUMAN_BLUE			2
#define TG_HUMAN_RED			3
#define TG_HUMAN_GROUP			4
#define TG_HUMAN_GROUP_BLUE		5
#define TG_HUMAN_GROUP_RED		6
#define TG_VEHICLE			7
#define TG_VEHICLE_BLUE			8
#define TG_VEHICLE_RED			9
#define TG_CAR				10
#define TG_CAR_BLUE			11
#define TG_CAR_RED			12
#define TG_TRUCK			13
#define TG_TRUCK_BLUE			14
#define TG_TRUCK_RED			15

#define COL_MONOCHROME			0
#define COL_RGB				1
#define COL_RGB_NORMALISED		2
#define COL_YUV				3
#define COL_HSV				4

#define PIX_MONOCHROME			0
#define PIX_RGB				1
#define PIX_RGBA			2
#define PIX_ABGR			3

#define BH_UNKNOWN			0
#define BH_HUMAN_STANDING		1
#define BH_HUMAN_WAKING			2
#define BH_HUMAN_RUNNING		3
#define BH_VEHICLE_PARKED		4
#define BH_VEHICLE_MOTION		5

/* event enumerations */
#define EV_UNCLASSIFIED			0
#define EV_RENDEZVOUS			1
#define EV_HVI_ENTRY			2
#define EV_HVI_EXIT			3
#define EV_LOITER			4
#define EV_CAR_ALARM			5
#define EV_CONFLICT			6

/* entity enumerations */
#define ID_OCU_1			0
#define ID_GUI_1			1
#define ID_VIS_1			2
#define ID_VIS_SEEDS			3
#define ID_SPU_1			4
#define ID_SPU_2			5
#define ID_SPU_3			6
#define ID_SPU_4			7
#define ID_SPU_5			8
#define ID_SPU_6			9
#define ID_SPU_7			10
#define ID_SPU_8			11
#define ID_SPU_9			12
#define ID_SPU_10			13

/* DIS fixed datum enumerations */
#define DIS_FX_COMMAND_ID		0
#define DIS_FX_SENSOR_ID		1
#define DIS_FX_TARGET_ID		2
#define DIS_FX_TARGET_BEHAV		3
#define DIS_FX_EVENT_ID			4

/* DIS variable datum enumerations */
#define DIS_VA_COMMAND_TARGET		0
#define DIS_VA_COMMAND_EVENT		1
#define DIS_VA_SENSOR_TEXT		2
#define DIS_VA_SENSOR_POS		3
#define DIS_VA_SENSOR_VEL		4
#define DIS_VA_SENSOR_FOV		5
#define DIS_VA_SENSOR_ROI		6
#define DIS_VA_SENSOR_FP		7
#define DIS_VA_SENSOR_LIMIT		8
#define DIS_VA_TARGET_TEXT		9
#define DIS_VA_TARGET_POS		10
#define DIS_VA_TARGET_VEL		11
#define DIS_VA_TARGET_IMG		12
#define DIS_VA_TARGET_TRAJ		13
#define DIS_VA_TARGET_TPOS		14
#define DIS_VA_TARGET_TIMG		15
#define DIS_VA_TARGET_TBEHAV		16
#define DIS_VA_EVENT_TEXT		17
#define DIS_VA_EVENT_POS		18
#define DIS_VA_EVENT_TARGET		19

#endif				/* __ENUMS_H__ */
