/*
 * print.c
 * Author: Yun-Ching Lee (ycl+@cmu.edu)
 * Copyright 1998 by Carnegie Mellon University
 * All rights reserved
 */

#include <unistd.h>
#include <stdio.h>
#include <stdlib.h>
#include "cmupa.h"
#include "print.h"

/*
 * print_cmupa() prints out cmupa data structure
 */

void 
print_cmupa(CMUPA data)
{
	u_short         h_bitmask = data.h_bitmask;
	long            j;	/* block counter */

	printf("packet_size = %ld\n", data.packet_size);
	printf("packet_id = %ld\n", data.packet_id);
	printf("scenario_id = %d\n", data.scenario_id);
	printf("source_id = %d\n", data.source_id);
	printf("dest_id = %d\n", data.dest_id);
	printf("time = %ld\n", data.time);
	if (data.num_commands <= 0 || data.command_block == NULL)
		CLEARBIT(h_bitmask, 0);
	if (data.num_sensors <= 0 || data.sensor_block == NULL)
		CLEARBIT(h_bitmask, 1);
	if (data.num_imageries <= 0 || data.imagery_block == NULL)
		CLEARBIT(h_bitmask, 2);
	if (data.num_targets <= 0 || data.target_block == NULL)
		CLEARBIT(h_bitmask, 3);
	if (data.num_events <= 0 || data.event_block == NULL)
		CLEARBIT(h_bitmask, 4);
	printf("h_bitmask = 0x%.2x\n", data.h_bitmask);

	/* print command section */
	if (TESTBIT(h_bitmask, 0)) {
		printf("num_commands = %d\n", data.num_commands);
		for (j = 0; j < data.num_commands; j++) {
			printf("command_block[%ld]\n", j);
			print_command(data.command_block[j]);
		}
	}
	/* print sensor section */
	if (TESTBIT(h_bitmask, 1)) {
		printf("num_sensors = %d\n", data.num_sensors);
		for (j = 0; j < data.num_sensors; j++) {
			printf("sensor_block[%ld]\n", j);
			print_sensor(data.sensor_block[j]);
		}
	}
	/* print imagery section */
	if (TESTBIT(h_bitmask, 2)) {
		printf("num_imageries = %d\n", data.num_imageries);
		for (j = 0; j < data.num_imageries; j++) {
			printf("imagery_block[%ld]\n", j);
			print_imagery(data.imagery_block[j]);
		}
	}
	/* print target section */
	if (TESTBIT(h_bitmask, 3)) {
		printf("num_targets = %d\n", data.num_targets);
		for (j = 0; j < data.num_targets; j++) {
			printf("target_block[%ld]\n", j);
			print_target(data.target_block[j]);
		}
	}
	/* print event section */
	if (TESTBIT(h_bitmask, 4)) {
		printf("num_events = %d\n", data.num_events);
		for (j = 0; j < data.num_events; j++) {
			printf("event_block[%ld]\n", j);
			print_event(data.event_block[j]);
		}
	}
}


void 
print_command(COMMAND_BLOCK command)
{
	u_long          c_bitmask = command.c_bitmask;
	long            k;

	printf("\tblock_size = %d\n", command.block_size);
	printf("\tcommand_id = %d\n", command.command_id);
	printf("\tcommand_type = %ld\n", command.command_type);
	printf("\tc_bitmask = 0x%.2x\n", (unsigned int) command.c_bitmask);

	/* encode targets fields */
	if (TESTBIT(c_bitmask, 0)) {
		printf("\ttargets field\n");
		printf("\t\tnum_targets = %d\n", command.num_targets);
		for (k = 0; k < command.num_targets; k++)
			printf("\t\ttarget_id[%ld] = %d\n", k, command.target_id[k]);
	}
	/* encode events fields */
	if (TESTBIT(c_bitmask, 1)) {
		printf("\tevents field\n");
		printf("\t\tnum_events = %d\n", command.num_events);
		for (k = 0; k < command.num_events; k++)
			printf("\t\tevent_id[%ld] = %d\n", k, command.event_id[k]);
	}
	/* encode bitmasks */
	if (TESTBIT(c_bitmask, 2)) {
		printf("\tpacket bitmasks\n");
		printf("\t\tc_bitmask = 0x%.2x\n", (unsigned int) command.packet_bitmask.c_bitmask);
		printf("\t\ts_bitmask = 0x%.2x\n", (unsigned int) command.packet_bitmask.s_bitmask);
		printf("\t\ti_bitmask = 0x%.2x\n", (unsigned int) command.packet_bitmask.i_bitmask);
		printf("\t\tt_bitmask = 0x%.2x\n", (unsigned int) command.packet_bitmask.r_bitmask);
		printf("\t\te_bitmask = 0x%.2x\n", (unsigned int) command.packet_bitmask.e_bitmask);
	}
	/* encode bandwidth */
	if (TESTBIT(c_bitmask, 3)) {
		printf("\tbandwidth\n");
		printf("\t\tbandwidth = %ld\n", command.bandwidth);
	}
}


void 
print_sensor(SENSOR_BLOCK sensor)
{
	u_long          s_bitmask = sensor.s_bitmask;
	long            k;

	printf("\tblock_size = %d\n", sensor.block_size);
	printf("\tsensor_id = %d\n", sensor.sensor_id);
	printf("\tsensor_type = %d\n", sensor.sensor_type);
	printf("\tsensor_status = %d\n", sensor.sensor_status);
	printf("\tlabel_len = %d\n", sensor.label_len);
	sensor.label[sensor.label_len] = (char) NULL;
	printf("\tlabel = %s\n", sensor.label);
	printf("\ts_bitmask 0x%.2x\n", (unsigned int) sensor.s_bitmask);

	/* encode LVCS position fields */
	if (TESTBIT(s_bitmask, 0)) {
		printf("\tlvcs_pos\n");
		printf("\t\tx = %f\n", sensor.lvcs_pos.x);
		printf("\t\ty = %f\n", sensor.lvcs_pos.y);
		printf("\t\tz = %f\n", sensor.lvcs_pos.z);
	}
	/* encode LVCS uncertainty fields */
	if (TESTBIT(s_bitmask, 1)) {
		printf("\tlvcs_err\n");
		printf("\t\tx = %f\n", sensor.lvcs_err.x);
		printf("\t\ty = %f\n", sensor.lvcs_err.y);
		printf("\t\tz = %f\n", sensor.lvcs_err.z);
	}
	/* encode LVCS velocity fields */
	if (TESTBIT(s_bitmask, 2)) {
		printf("\tlvcs_vel\n");
		printf("\t\tx = %f\n", sensor.lvcs_vel.x);
		printf("\t\ty = %f\n", sensor.lvcs_vel.y);
		printf("\t\tz = %f\n", sensor.lvcs_vel.z);
	}
	/* encode GPS position fields */
	if (TESTBIT(s_bitmask, 3)) {
		printf("\tgps_pos\n");
		printf("\t\tlat = %ld\n", sensor.gps_pos.lat);
		printf("\t\tlon = %ld\n", sensor.gps_pos.lon);
		printf("\t\talt = %f\n", sensor.gps_pos.alt);
	}
	/* encode GPS uncertainty fields */
	if (TESTBIT(s_bitmask, 4)) {
		printf("\tgps_err\n");
		printf("\t\tlat = %ld\n", sensor.gps_err.lat);
		printf("\t\tlon = %ld\n", sensor.gps_err.lon);
		printf("\t\talt = %f\n", sensor.gps_err.alt);
	}
	/* encode GPS velocity fields */
	if (TESTBIT(s_bitmask, 5)) {
		printf("\tgps_vel\n");
		printf("\t\tlat = %f\n", sensor.gps_vel.lat);
		printf("\t\tlon = %f\n", sensor.gps_vel.lon);
		printf("\t\talt = %f\n", sensor.gps_vel.alt);
	}
	/* encode mount orientation fields */
	if (TESTBIT(s_bitmask, 6)) {
		printf("\tmount_pos\n");
		printf("\t\ttheta = %ld\n", sensor.mount_pos.theta);
		printf("\t\tphi = %ld\n", sensor.mount_pos.phi);
		printf("\t\trho = %ld\n", sensor.mount_pos.rho);
	}
	/* encode mount orientation uncertainty fields */
	if (TESTBIT(s_bitmask, 7)) {
		printf("\tmount_err\n");
		printf("\t\ttheta = %ld\n", sensor.mount_err.theta);
		printf("\t\tphi = %ld\n", sensor.mount_err.phi);
		printf("\t\trho = %ld\n", sensor.mount_err.rho);
	}
	/* encode mount velocity fields */
	if (TESTBIT(s_bitmask, 8)) {
		printf("\tmount_vel\n");
		printf("\t\ttheta = %f\n", sensor.mount_vel.theta);
		printf("\t\tphi = %f\n", sensor.mount_vel.phi);
		printf("\t\trho = %f\n", sensor.mount_vel.rho);
	}
	/* encode image size fields */
	if (TESTBIT(s_bitmask, 9)) {
		printf("\timage_size\n");
		printf("\t\tu = %d\n", sensor.image_size.u);
		printf("\t\tv = %d\n", sensor.image_size.v);
	}
	/* encode FOV fields */
	if (TESTBIT(s_bitmask, 10)) {
		printf("\tfov\n");
		printf("\t\th_angle = %ld\n", sensor.fov.h_angle);
		printf("\t\tv_angle = %ld\n", sensor.fov.v_angle);
	}
	/* encode sensor internal params fields */
	if (TESTBIT(s_bitmask, 11)) {
		printf("\tsen_pars\n");
		printf("\t\tiris = %d\n", sensor.sen_pars.iris);
		printf("\t\tfocus = %d\n", sensor.sen_pars.focus);
		printf("\t\tu_centre = %d\n", sensor.sen_pars.u_centre);
		printf("\t\tv_centre = %d\n", sensor.sen_pars.v_centre);
		printf("\t\tdistortion = %f\n", sensor.sen_pars.distortion);
	}
	/* encode pan/tilt/zoom position fields */
	if (TESTBIT(s_bitmask, 12)) {
		printf("\tptz_pos\n");
		printf("\t\tpan = %ld\n", sensor.ptz_pos.pan);
		printf("\t\ttilt = %ld\n", sensor.ptz_pos.tilt);
		printf("\t\ttwist = %ld\n", sensor.ptz_pos.twist);
		printf("\t\tzoom = %f\n", sensor.ptz_pos.zoom);
	}
	/* encode pan/tilt/zoom velocity fields */
	if (TESTBIT(s_bitmask, 13)) {
		printf("\tptz_vel\n");
		printf("\t\tpan = %f\n", sensor.ptz_vel.pan);
		printf("\t\ttilt = %f\n", sensor.ptz_vel.tilt);
		printf("\t\ttwist = %f\n", sensor.ptz_vel.twist);
		printf("\t\tzoom = %f\n", sensor.ptz_vel.zoom);
	}
	/* encode pan/tilt/zoom region of interest fields */
	if (TESTBIT(s_bitmask, 14)) {
		printf("\tptz_roi\n");
		printf("\t\tnum_pos = %d\n", sensor.ptz_roi.num_pos);
		for (k = 0; k < sensor.ptz_roi.num_pos; k++) {
			printf("\t\tptz_coord[%ld]\n", k);
			printf("\t\t\tpan = %ld\n", sensor.ptz_roi.ptz_coord[k].pan);
			printf("\t\t\ttilt = %ld\n", sensor.ptz_roi.ptz_coord[k].tilt);
			printf("\t\t\ttwist = %ld\n", sensor.ptz_roi.ptz_coord[k].twist);
			printf("\t\t\tzoom = %f\n", sensor.ptz_roi.ptz_coord[k].zoom);
		}
	}
	/* encode LVCS region of interest fields */
	if (TESTBIT(s_bitmask, 15)) {
		printf("\tlvcs_roi\n");
		printf("\t\tnum_pos = %d\n", sensor.lvcs_roi.num_pos);
		for (k = 0; k < sensor.lvcs_roi.num_pos; k++) {
			printf("\t\tlvcs_coord[%ld]\n", k);
			printf("\t\t\tx = %f\n", sensor.lvcs_roi.lvcs_coord[k].x);
			printf("\t\t\ty = %f\n", sensor.lvcs_roi.lvcs_coord[k].y);
			printf("\t\t\tz = %f\n", sensor.lvcs_roi.lvcs_coord[k].z);
		}
	}
	/* encode camera footprlong in ptz */
	if (TESTBIT(s_bitmask, 16)) {
		printf("\tptz_fp\n");
		printf("\t\ttop_left.theta = %ld\n", sensor.ptz_fp.top_left.theta);
		printf("\t\ttop_left.phi = %ld\n", sensor.ptz_fp.top_left.phi);
		printf("\t\tbottom_right.theta = %ld\n", sensor.ptz_fp.bottom_right.theta);
		printf("\t\tbottom_right.phi = %ld\n", sensor.ptz_fp.bottom_right.phi);
	}
	/* encode camera footprlong in LVCS */
	if (TESTBIT(s_bitmask, 17)) {
		printf("\tlvcs_fp\n");
		printf("\t\tnum_pos = %d\n", sensor.lvcs_fp.num_pos);
		for (k = 0; k < sensor.lvcs_fp.num_pos; k++) {
			printf("\t\tlvcs_coord[%ld]\n", k);
			printf("\t\t\tx = %f\n", sensor.lvcs_fp.lvcs_coord[k].x);
			printf("\t\t\ty = %f\n", sensor.lvcs_fp.lvcs_coord[k].y);
			printf("\t\t\tz = %f\n", sensor.lvcs_fp.lvcs_coord[k].z);
		}
	}
	/* encode camera limit in ptz format */
	if (TESTBIT(s_bitmask, 18)) {
		printf("\tptz_lim\n");
		printf("\t\ttop_left.theta = %ld\n", sensor.ptz_lim.top_left.theta);
		printf("\t\ttop_left.phi = %ld\n", sensor.ptz_lim.top_left.phi);
		printf("\t\tbottom_right.theta = %ld\n", sensor.ptz_lim.bottom_right.theta);
		printf("\t\tbottom_rgith.phi = %ld\n", sensor.ptz_lim.bottom_right.phi);
	}
	/* encode camera limit in LVCS format */
	if (TESTBIT(s_bitmask, 19)) {
		printf("\tlvcs_lim\n");
		printf("\t\tnum_pos = %d\n", sensor.lvcs_lim.num_pos);
		for (k = 0; k < sensor.lvcs_lim.num_pos; k++) {
			printf("\t\tlvcs_coord[%ld]\n", k);
			printf("\t\t\tx = %f\n", sensor.lvcs_lim.lvcs_coord[k].x);
			printf("\t\t\ty = %f\n", sensor.lvcs_lim.lvcs_coord[k].y);
			printf("\t\t\tz = %f\n", sensor.lvcs_lim.lvcs_coord[k].z);
		}
	}
}

void 
print_imagery(IMAGERY_BLOCK imagery)
{
	u_long          i_bitmask = imagery.i_bitmask;

	printf("\tblock_size = %ld\n", imagery.block_size);
	printf("\timagery_id = %d\n", imagery.imagery_id);
	printf("\timagery_type = %d\n", imagery.imagery_type);
	printf("\tlabel_len = %d\n", imagery.label_len);
	imagery.label[imagery.label_len] = (char) NULL;
	printf("\tlabel = %s\n", imagery.label);
	printf("\ti_bitmask = %.2x\n", (unsigned int) i_bitmask);

	/* encode image data */
	if (TESTBIT(i_bitmask, 0)) {
		printf("\timagery_block\n");
		printf("\t\ttime = %ld\n", imagery.imagery.time);
		printf("\t\twidth = %d\n", imagery.imagery.width);
		printf("\t\theight = %d\n", imagery.imagery.height);
		printf("\t\tformat = %d\n", imagery.imagery.format);
		printf("\t\tsize = %ld\n", imagery.imagery.size);
		/*
		 * we don't really want to print binary data
		printf("\t\tdata = ");
		write(STDOUT_FILENO, imagery->imagery.data, imagery->imagery.size); printf("\n");
		 */
	}
	/* encode video data */
	if (TESTBIT(i_bitmask, 1)) {
		printf("\tvideo_block\n");
		printf("\t\tstart_time = %ld\n", imagery.video.start_time);
		printf("\t\tend_time = %ld\n", imagery.video.end_time);
		printf("\t\twidth = %d\n", imagery.video.width);
		printf("\t\theight = %d\n", imagery.video.height);
		printf("\t\tformat = %d\n", imagery.video.format);
		printf("\t\tsize = %ld\n", imagery.video.size);
		/*
		 * we don't really want to print binary data
		printf("\t\tdata = ");
		write(STDOUT_FILENO, imagery->imagery.data, imagery->video.size); printf("\n");
		 */
	}
}

void 
print_target(TARGET_BLOCK target)
{
	u_long          r_bitmask = target.r_bitmask;
	u_long          t_bitmask = target.t_bitmask;
	long            traj_len;
	long            k;
	long            l;
	long            m;

	printf("\tblock_size = %d\n", target.block_size);
	printf("\ttarget_id = %d\n", target.target_id);
	printf("\ttarget_type = %d\n", target.target_type);
	printf("\tlabel_len = %d\n", target.label_len);
	target.label[target.label_len] = (char) NULL;
	printf("\tlabel = %s\n", target.label);
	printf("\tr_bitmask = %.2x\n", (unsigned int) target.r_bitmask);
	printf("\tt_bitmask = %.2x\n", (unsigned int) target.t_bitmask);

	/* encode target position in img coords */
	if (TESTBIT(r_bitmask, 0)) {
		printf("\timg_pos\n");
		printf("\t\tu = %d\n", target.img_pos.u);
		printf("\t\tv = %d\n", target.img_pos.v);
	}
	/* encode target position in LVCS coords */
	if (TESTBIT(r_bitmask, 1)) {
		printf("\tlvcs_pos\n");
		printf("\t\tx = %f\n", target.lvcs_pos.x);
		printf("\t\ty = %f\n", target.lvcs_pos.y);
		printf("\t\tz = %f\n", target.lvcs_pos.z);
	}
	/* encode LVCS target uncertainty (covariance) */
	if (TESTBIT(r_bitmask, 2)) {
		printf("\tlvcs_err\n");
		for (k = 0; k < 3; k++) {
			printf("\t");
			for (l = 0; l < 3; l++)
				printf("\t%f", target.lvcs_err.C[k][l]);
			printf("\n");
		}
	}
	/* encode target velocity in image coords */
	if (TESTBIT(r_bitmask, 3)) {
		printf("\timg_vel\n");
		printf("\t\tu = %f\n", target.img_vel.u);
		printf("\t\tv = %f\n", target.img_vel.v);
	}
	/* encode target velocity in LVCS coords */
	if (TESTBIT(r_bitmask, 4)) {
		printf("\tlvcs_vel\n");
		printf("\t\tx = %f\n", target.lvcs_vel.x);
		printf("\t\ty = %f\n", target.lvcs_vel.y);
		printf("\t\tz = %f\n", target.lvcs_vel.z);
	}
	/* encode target bounding box */
	if (TESTBIT(r_bitmask, 5)) {
		printf("\tbounding_box\n");
		printf("\t\ttop_left.u = %d\n", target.bounding_box.top_left.u);
		printf("\t\ttop_left.v = %d\n", target.bounding_box.top_left.v);
		printf("\t\tbottom_right.u = %d\n", target.bounding_box.bottom_right.u);
		printf("\t\tbottom_right.v = %d\n", target.bounding_box.bottom_right.v);
	}
	/* encode target skeleton */
	if (TESTBIT(r_bitmask, 6)) {
		printf("\tskeleton\n");
		printf("\t\tcentroid.u = %d\n", target.skeleton.centroid.u);
		printf("\t\tcentroid.v = %d\n", target.skeleton.centroid.v);
		printf("\t\tlength = %d\n", target.skeleton.length);
		for (k = 0; k < target.skeleton.length; k++) {
			printf("\t\textremal[%ld].u = %d\n", k, target.skeleton.extremal[k].u);
			printf("\t\textremal[%ld].v = %d\n", k, target.skeleton.extremal[k].v);
		}
	}
	/* encode target bundary */
	if (TESTBIT(r_bitmask, 7)) {
		printf("\tboundary\n");
		printf("\t\tlength = %d\n", target.boundary.length);
		for (k = 0; k < target.boundary.length; k++) {
			printf("\t\tvalue[%ld].u = %d\n", k, target.boundary.value[k].u);
			printf("\t\tvalue[%ld].v = %d\n", k, target.boundary.value[k].v);
		}
	}
	/* encode target size */
	if (TESTBIT(r_bitmask, 8))
		printf("\ttarget_size = %d\n", target.target_size);

	/* encode target color histogram */
	if (TESTBIT(r_bitmask, 9)) {
		printf("\tcolour_histogram\n");
		printf("\t\tcolour_space = %d\n", target.histogram.colour_space);
		printf("\t\tchannel_1_bins = %d\n", target.histogram.channel_1_bins);
		printf("\t\tchannel_2_bins = %d\n", target.histogram.channel_2_bins);
		printf("\t\tchannel_3_bins = %d\n", target.histogram.channel_3_bins);
		/* do we really want to print out the histogram? */
		/*
		 * for(k=0; k<target.histogram.channel_1_bins; k++)
		 * printf("\t\t\tchannel_1[%d] = %d\n", k,
		 * target.histogram.channel_1[k]); for(k=0;
		 * k<target.histogram.channel_2_bins; k++)
		 * printf("\t\t\tchannel_2[%d] = %d\n", k,
		 * target.histogram.channel_2[k]); for(k=0;
		 * k<target.histogram.channel_3_bins; k++)
		 * printf("\t\t\tchannel_3[%d] = %d\n", k,
		 * target.histogram.channel_3[k]);
		 */
	}
	/* encode image template */
	if (TESTBIT(r_bitmask, 10)) {
		printf("\ttemplate\n");
		printf("\t\tsize.u = %d\n", target.temp.size.u);
		printf("\t\tsize.v = %d\n", target.temp.size.v);
		printf("\t\tpacking = %d\n", target.temp.packing);
		/*
		 * we don't really want to print the template
		printf("\t\tdata = ");
		write(STDOUT_FILENO, target.temp.data, target.temp.size.u*target.temp.size.v);
		printf("\n");
		 */
	}
	/* encode image depth map */
	if (TESTBIT(r_bitmask, 11)) {
		printf("\tdepth_map\n");
		printf("\t\tsize.u = %d\n", target.depth_map.size.u);
		printf("\t\tsize.v = %d\n", target.depth_map.size.v);
		printf("\t\tpacking = %d\n", target.depth_map.packing);
		/*
		 * we don't really want to print binary codes
		printf("\t\tdata = ");
		write(STDOUT_FILENO, target->depth_map.data, target->depth_map.size.u*target->depth_map.size.v);
		printf("\n");
		 */
	}
	/* encode depth map resolution table */
	if (TESTBIT(r_bitmask, 12)) {
		printf("\tdepth_res\n");
		printf("\t\tdisparities = %d\n", target.depth_res.disparities);
		for (k = 0; k < target.depth_res.disparities; k++)
			printf("\t\tres_table[%ld] = %f\n", k, target.depth_res.res_table[k]);
	}
	/* encode target behaviour */
	if (TESTBIT(r_bitmask, 13))
		printf("\tbehav = %d\n", target.behav);

	/* encode trajectory on/off length/timestamp fields */
	if (TESTBIT(t_bitmask, 0)) {
		printf("\ttrajectory\n");
		printf("\t\ttraj_len = %d\n", target.on_off.trajectory_len);
		traj_len = target.on_off.trajectory_len;
		/* timestamp */
		for (k = 0; k < traj_len; k++)
			printf("\t\ttime[%ld] = %ld\n", k, target.on_off.time[k]);

		/* encode trajectory in img coords */
		if (TESTBIT(t_bitmask, 1))
			for (k = 0; k < traj_len; k++) {
				printf("\t\tttraj_img_pos[%ld]\n", k);
				printf("\t\t\tu = %d\n", target.traj_img_pos[k].u);
				printf("\t\t\tv = %d\n", target.traj_img_pos[k].v);
			}

		/* encode trajectory in LVCS coords */
		if (TESTBIT(t_bitmask, 2))
			for (k = 0; k < traj_len; k++) {
				printf("\t\ttraj_lvcs_pos[%ld]\n", k);
				printf("\t\t\tx = %f\n", target.traj_lvcs_pos[k].x);
				printf("\t\t\ty = %f\n", target.traj_lvcs_pos[k].y);
				printf("\t\t\tz = %f\n", target.traj_lvcs_pos[k].z);
			}


		/* encode trajectory covariance matrix */
		if (TESTBIT(t_bitmask, 3))
			for (k = 0; k < traj_len; k++) {
				printf("\t\ttraj_lvcs_err[%ld]\n", k);
				for (l = 0; l < 3; l++) {
					printf("\t\t");
					for (m = 0; m < 3; m++)
						printf("\t%f", target.traj_lvcs_err[k].C[l][m]);
					printf("\n");
				}
			}

		/* encode trajectory of bounding box */
		if (TESTBIT(t_bitmask, 4))
			for (k = 0; k < traj_len; k++) {
				printf("\t\ttraj_bounding_box[%ld]\n", k);
				printf("\t\t\ttop_left.u = %d\n", target.traj_bound_box[k].top_left.u);
				printf("\t\t\ttop_left.v = %d\n", target.traj_bound_box[k].top_left.v);
				printf("\t\t\tbottom_right.u = %d\n", target.traj_bound_box[k].bottom_right.u);
				printf("\t\t\tbottom_right.v = %d\n", target.traj_bound_box[k].bottom_right.v);
			}

		/* encode trajectory of skeleton */
		if (TESTBIT(t_bitmask, 5))
			for (k = 0; k < traj_len; k++) {
				printf("\t\ttraj_skeleton[%ld]\n", k);
				printf("\t\t\tcentroid.u = %d\n", target.traj_skeleton[k].centroid.u);
				printf("\t\t\tcentroid.v = %d\n", target.traj_skeleton[k].centroid.v);
				printf("\t\t\tlength = %d\n", target.traj_skeleton[k].length);
				for (l = 0; l < target.traj_skeleton[k].length; l++) {
					printf("\t\t\textremal[%ld].u = %d\n", l, target.traj_skeleton[k].extremal[l].u);
					printf("\t\t\textremal[%ld].v = %d\n", l, target.traj_skeleton[k].extremal[l].v);
				}
			}

		/* encode trajectory of boundary */
		if (TESTBIT(t_bitmask, 6))
			for (k = 0; k < traj_len; k++) {
				printf("\t\ttraj_boundary[%ld]\n", k);
				printf("\t\t\tlength = %d\n", target.traj_boundary[k].length);
				for (l = 0; l < target.traj_boundary[k].length; l++) {
					printf("\t\t\tvalue[%ld].u = %d\n", k, target.traj_boundary[k].value[l].u);
					printf("\t\t\tvalue[%ld].v = %d\n", k, target.traj_boundary[k].value[l].v);
				}
			}

		/* encode trajectory of target size */
		if (TESTBIT(t_bitmask, 7))
			for (k = 0; k < traj_len; k++)
				printf("\t\ttraj_size[%ld] = %d\n", k, target.traj_size[k]);

		/* encode trajectory of color histogram */
		if (TESTBIT(t_bitmask, 8))
			for (k = 0; k < traj_len; k++) {
				printf("\t\ttraj_histogram[%ld]\n", k);
				printf("\t\t\tcolour_space = %d\n", target.traj_histogram[k].colour_space);
				printf("\t\t\tchannel_1_bins = %d\n", target.traj_histogram[k].channel_1_bins);
				printf("\t\t\tchannel_2_bins = %d\n", target.traj_histogram[k].channel_2_bins);
				printf("\t\t\tchannel_3_bins = %d\n", target.traj_histogram[k].channel_3_bins);
				/*
				 * don't print the bins for(l=0;
				 * l<target->traj_histogram[k].channel_1_bins;
				 *  l++) printf("\t\t\tchannel_1[%ld] =
				 * %d\n", l,
				 * target.traj_histogram[k].channel_1[l]);
				 * for(l=0;
				 * l<target->traj_histogram[k].channel_2_bins;
				 *  l++) printf("\t\t\tchannel_2[%ld] =
				 * %d\n", l,
				 * target.traj_histogram[k].channel_2[l]);
				 * for(l=0;
				 * l<target->traj_histogram[k].channel_3_bins;
				 *  l++) printf("\t\t\tchannel_3[%ld] =
				 * %d\n", l,
				 * target.traj_histogram[k].channel_3[l]);
				 */
			}

		/* encode trajectory of image template */
		if (TESTBIT(t_bitmask, 9))
			for (k = 0; k < traj_len; k++) {
				printf("\t\ttraj_template[%ld]\n", k);
				printf("\t\t\tsize.u = %d\n", target.traj_template[k].size.u);
				printf("\t\t\tsize.v = %d\n", target.traj_template[k].size.v);
				printf("\t\t\tpacking = %d\n", target.traj_template[k].packing);
				/*
				 * we don't want to print this
				printf("\t\t\tdata = ");
				write(STDOUT_FILENO, target.traj_template[k].data, target.traj_template[k].size.u*target.traj_template[k].size.v);
				printf("\n");
				 */
			}

		/* encode trajectory depth map */
		if (TESTBIT(t_bitmask, 10))
			for (k = 0; k < traj_len; k++) {
				printf("\t\ttraj_depth_map[%ld]\n", k);
				printf("\t\t\tsize.u = %d\n", target.traj_depth_map[k].size.u);
				printf("\t\t\tsize.v = %d\n", target.traj_depth_map[k].size.v);
				printf("\t\t\tpacking = %d\n", target.traj_depth_map[k].packing);
				/*
				 * we don't want to print binary stuff
				printf("\t\t\tdata = ");
				write(STDOUT_FILENO, target->traj_depth_map[k].data, target->traj_depth_map[k].size.u*target->traj_depth_map[k].size.v);
				printf("\n");
				 */
			}

		/* encode trajectory behaviour */
		if (TESTBIT(t_bitmask, 11))
			for (k = 0; k < traj_len; k++)
				printf("\t\ttraj_behav[%ld] = %d\n", k, target.traj_behav[k]);
	}
}

void 
print_event(EVENT_BLOCK event)
{
	u_long          e_bitmask = event.e_bitmask;
	long            k;

	printf("\tblock_size = %d\n", event.block_size);
	printf("\tevent_id = %d\n", event.event_id);
	printf("\tevent_type = %d\n", event.event_type);
	printf("\tlabel_len = %d\n", event.label_len);
	event.label[event.label_len] = (char) NULL;
	printf("\tlabel = %s\n", event.label);
	printf("\te_bitmask = %.2x\n", (unsigned int) event.e_bitmask);

	/* encode event position in img coords */
	if (TESTBIT(e_bitmask, 0)) {
		printf("\tevent_img\n");
		printf("\t\tpos.u = %d\n", event.event_img_pos.u);
		printf("\t\tpos.v = %d\n", event.event_img_pos.v);
		printf("\t\trange = %f\n", event.event_img_range);
		printf("\t\ttime = %ld\n", event.event_img_time);
	}
	/* encode event position in LVCS coords */
	if (TESTBIT(e_bitmask, 1)) {
		printf("\tevent_lvcs\n");
		printf("\t\tpos.x = %f\n", event.event_lvcs_pos.x);
		printf("\t\tpos.y = %f\n", event.event_lvcs_pos.y);
		printf("\t\tpos.z = %f\n", event.event_lvcs_pos.z);
		printf("\t\ttime = %ld\n", event.event_lvcs_time);
	}
	/* encode target list */
	if (TESTBIT(e_bitmask, 2)) {
		printf("\ttarget_list\n");
		printf("\t\tnum_targets = %d\n", event.num_targets);
		for (k = 0; k < event.num_targets; k++)
			printf("\t\ttarget_id[%ld] = %d\n", k, event.target_id[k]);
	}
}
