/*
 * //------------------------------------
 * //  STRUCTURES.H
 * //  by Alan Lipton. May 1998
 * //  Copyright Carnegie Mellon University
 * //  All rights reserved
 * //------------------------------------
 */

#ifndef	__STRUCTURES_H__
#define __STRUCTURES_H__

typedef struct _image_point {
	__int16         u;
	__int16         v;
}               IMAGE_POINT;

typedef struct _image_velocity {
	double          u;
	double          v;
}               IMAGE_VELOCITY;

typedef struct _bounding_box {
	IMAGE_POINT     top_left;
	IMAGE_POINT     bottom_right;
}               BOUNDING_BOX;

typedef struct _motion_roi {
	BOUNDING_BOX    b_box;
	IMAGE_POINT     centroid;
	__int16         size;
	long            sum_x, sum_y;
}               MOTION_ROI;

typedef struct _template {
	IMAGE_POINT     size;
	IMAGE_POINT     offset;
	__int16         packing;
	unsigned char   data[TEMPLATE_MEM];
	bool            motion[TEMPLATE_MEM];
}               TEMPLATE;

typedef struct _colour_histogram {
	double          channel_1[HISTOGRAM_SIZE];
	double          channel_2[HISTOGRAM_SIZE];
	double          channel_3[HISTOGRAM_SIZE];
}               COLOUR_HISTOGRAM;

typedef struct _boundary {
	__int16         length;
	IMAGE_POINT    *value;
}               BOUNDARY;

typedef struct _skeleton {
	IMAGE_POINT     centroid;
	__int16         length;
	IMAGE_POINT     extremal[SKELETON_SIZE];
}               SKELETON;

typedef struct _classification {
	__int16         present;
	__int16         absent;
	__int16         unclassified;
	__int16         clutter_reject;
	__int16         human;
	__int16         human_group;
	__int16         vehicle;
	__int16         car;
	__int16         truck;
}               CLASSIFICATION;


typedef struct _sensor_pose {
	double          lat, lon, alt;
	double          rho, theta, phi;
	double          pan, tilt;
	double          h_fov, v_fov;
}               SENSOR_POSE;


typedef struct _sphere_point {
	double          azim, elev;
}               SPHERE_POINT;

typedef struct _target {
	/* HEADER INFORMATION */
	__int16         ID;
	__int16         type;
	__int16         status;
	bool            active;
	bool            occluded;
	bool            visible;

	IMAGE_POINT     origin;

	CLASSIFICATION  classification_histogram;

	/* TRAJECTORY INFORMATION */
	__int32         trajectory_mask;
	__int16         trajectory_length;
	__int16         current_time;
	DWORD           time[TRAJECTORY_LENGTH];

	/* DATA STORAGE */
	__int16         current_position;
	IMAGE_POINT     position[TRAJECTORY_LENGTH];
	SENSOR_POSE     pose[TRAJECTORY_LENGTH];
	SPHERE_POINT    SpherePosition[TRAJECTORY_LENGTH];
	__int16         current_velocity;
	IMAGE_VELOCITY  velocity[TRAJECTORY_LENGTH];
	__int16         current_acceleration;
	IMAGE_VELOCITY  acceleration[TRAJECTORY_LENGTH];
	__int16         current_b_box;
	BOUNDING_BOX    b_box[TRAJECTORY_LENGTH];
	__int16         current_chip;
	TEMPLATE        chip[1];
	__int16         current_histogram;
	COLOUR_HISTOGRAM histogram[TRAJECTORY_LENGTH];
	__int16         current_boundary;
	BOUNDARY        boundary[TRAJECTORY_LENGTH];
	__int16         current_size;
	__int16         size[TRAJECTORY_LENGTH];
	__int16         current_skeleton;
	SKELETON        skeleton[TRAJECTORY_LENGTH];
}               TARGET;

#endif				/* __STRUCTURES_H__ */
