/*
 * OCU_loop.c - the main processing loop of OCU
 * Author: Yun-Ching Lee (ycl+@cmu.edu)
 * Copyright 1998 by Carnegie Mellon University
 * All rights reserved
 */

#include "OCU98.h"

void 
OCU_loop(void)
{
	get_new_client();
	get_new_data();
	/*
	 * handoff(); update_SPU(); update_GUI(); send_DIS();
	 */
}

void 
get_new_client(void)
{
	int             newsock;
	CMUPA		inpacket;

	if (new_client(sock)) {
		newsock = accept_client(sock);
		printf("got a new client\n");
		/* put handshaking stuff here */
		/* the first packet should have just 1 command block with
		   CM_I_AM_ALIVE */
		receive_packet(newsock, &inpacket);
		print_cmupa(inpacket);
		if(inpacket.command_block[0].command_type != CM_I_AM_ALIVE) {
			shutdown(newsock, 2);
			close(newsock);
			printf("get_new_client: did not receive CM_I_AM_ALIVE\n");
		}

		/* should receive supported bitmasks and sensor status now */
		receive_packet(newsock, &inpacket);
		print_cmupa(inpacket);
		if(inpacket.command_block[0].command_type != CM_SET_PACKET_DESCRIPTION) {
			shutdown(newsock, 2);
			close(newsock);
			printf("get_new_client: did not receive CM_SET_PACKET_DESCRIPTION\n");
		}
		if(inpacket.sensor_block <= 0) {
			shutdown(newsock, 2);
			close(newsock);
			printf("get_new_client: did not receive a sensor block\n");
		}

		/* identify the new socket from its source */
		spu_sock[inpacket.source_id] = newsock;
		memcpy(&spu_packet, &inpacket, sizeof(CMUPA));
	}
}

void 
get_new_data(void)
{
	int             i;

	for (i = 0; i < no_clients; i++) {
		while (data_available(spu_sock[i])) {
			printf("get_new_data: got a new packet on socket %ld\n", spu_sock[i]);
			receive_packet(spu_sock[i], &spu_packet[i]);
			print_cmupa(spu_packet[i]);
		}
	}
}

void 
receive_packet(int sock, CMUPA * packet)
{
	int             length;
	int             position;
	u_long          remain;
	char            buffer[MAX_BUFFER_LENGTH];
	int             retval;

	length = read(sock, buffer, 4);
	if (length != 4) {
		printf("receive_packet: read error\n");
		return;
	}
	buftoint32(buffer, &remain);
	remain -= 4;
	position = 4;

	while (remain > 0) {
		length = read(sock, buffer + position, remain);
		if (length > 0) {
			remain -= length;
			position += length;
		}
	}

	retval = unpack_cmupa(buffer, packet);
	if (retval == ERROR) {
		printf("receive_packet: unpack_cmupa error\n");
	}
}
