/*
 * socket.c - socket operations
 * Author: Yun-Ching Lee (ycl+@cmu.edu)
 * Copyright 1998 by Carnegie Mellon University
 * All rights reserved
 */

#include "OCU98.h"
#include "socket.h"

/*
 * all functions here return 1 or length of data transmitted on success, and
 * 0 on failure.
 */

int 
net_startup(void)
{
	struct sockaddr_in sai;
	struct linger   ling;

	sai.sin_port = htons(PORT);
	sai.sin_family = AF_INET;
	sai.sin_addr.s_addr = 0;/* any host */

	sock = socket(AF_INET, SOCK_STREAM, 0);
	if (sock == ERROR) {
		printf("net_startup: socket error\n");
		return (0);
	}
	if (bind(sock, (struct sockaddr *) & sai, sizeof(struct sockaddr_in)) == ERROR) {
		printf("net_startup: bind error\n");
		return (0);
	}
	if (listen(sock, SOMAXCONN) == ERROR) {
		printf("net_startup: listen error\n");
		return (0);
	}
	ling.l_onoff = 1;
	ling.l_linger = 0;
	if (setsockopt(sock, SOL_SOCKET, SO_LINGER, (const char *) &ling, sizeof(struct linger)) == ERROR) {
		printf("net_startup: setsockopt error\n");
		return (0);
	}
	return (1);
}

int 
net_shutdown(void)
{
	int             i;

	close(sock);
	for (i = 0; i < no_clients; i++)
		close(spu_sock[i]);

	return (1);
}

int 
new_client(int sock)
{
	fd_set          rfds;
	struct timeval  time_out;
	int             selval;

	time_out.tv_sec = 0;
	time_out.tv_usec = 0;

	FD_ZERO(&rfds);
	FD_SET(sock, &rfds);

	selval = select(sock + 1, &rfds, NULL, NULL, &time_out);
	if (selval == ERROR) {
		printf("new_client: select error\n");
		selval = 0;
	}
	return (selval);
}

int 
accept_client(int sock)
{
	int             sockfd;
	struct linger   ling;

	sockfd = accept(sock, NULL, NULL);
	if (sockfd == ERROR) {
		printf("accept_client: accept error\n");
		return (0);
	}
	ling.l_onoff = 1;
	ling.l_linger = 0;

	if (setsockopt(sockfd, SOL_SOCKET, SO_LINGER, (const char *) &ling, sizeof(struct linger)) == ERROR) {
		printf("accept_client: setsockopt error\n");
	}
	return (sockfd);
}

int 
data_available(int sock)
{
	fd_set          rfds;
	struct timeval  time_out;
	int             selval;

	time_out.tv_sec = 0;
	time_out.tv_usec = 0;

	FD_ZERO(&rfds);
	FD_SET(sock, &rfds);

	selval = select(sock + 1, &rfds, NULL, NULL, &time_out);
	if (selval == ERROR) {
		printf("data_available: select error\n");
		return (0);
	}
	if (FD_ISSET(sock, &rfds))
		return (1);
	else
		return (0);
}
